//
// vfive: Vector Field Interactive Visualization Environment
//
// anicont.c++
//
//                by Ohno 2009.05.8
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 



#include "vfive.h"

extern Data *data;
extern Isosurf *isosurf;
extern OrthoSlicer *orthoSlicer[3];	// xy, yz, and zx planes.
extern cave_volrend *volumerendering;
extern Tracer *tracer;
extern Snow *snow;
extern Hotaru *hotaru;
extern Probe *probe;
extern Recalc *recalc;
extern scalefHandle *scalefh;

extern GLUquadricObj *cyl[CAVE_MAX_WALLS];                   // cylinder

const float MIN_INTVAL = 0.5;
const float MAX_INTVAL = 5.0;

AnimControl::AnimControl(char *file)
{
  int n = 0;
  int found = v5_get_int(file, "ntime", &n);

  if(found && n > MAX_DLIST_TIME) dlist_mode = false;
  	else dlist_mode = true;

  if(n == 0) n=1;
  dmade4anim = new int [n];
  for(int i=0;i<n;i++) dmade4anim[i] = 0;

  current_time = 0;
  anim_on = false;
  time_of_start = 0;
  all_set = false;
  num_threads = CAVENumPipes() + 1; // +1 means computation threads
  //printf("num threads = %d\n",num_threads); 
  TimeOfLoadedData = 0;
  prev_frame = CAVEGetFrameNumber(); 
  prev_time  = CAVEGetTime();
  recover_flag = 0;
  intval = 3.0;
  ani_recalc = 0;  
  //GUI
  vert_level_bar_rad_max = 0.2;
  vert_level_bar_height = 2.0;

//  work_level = intval;
  work_level = 1.0/intval;
  
//  work_vmin = MIN_INTVAL;
//  work_vmax = MAX_INTVAL;
  work_vmax = 1.0/MIN_INTVAL;
  work_vmin = 1.0/MAX_INTVAL;
//  work_aver = (MAX_INTVAL - MIN_INTVAL) / 2.0;
  work_aver = (1.0/MIN_INTVAL - 1.0/MAX_INTVAL) / 2.0;
  work_dval = work_vmax - work_vmin;

  float abs_vmax = fabs (work_vmax);
  float abs_vmin = fabs (work_vmin);
  float abs_larger = abs_vmax > abs_vmin ? abs_vmax : abs_vmin;
  vert_drad_dlevel = vert_level_bar_rad_max / abs_larger;

  snapstate = new Snapstate;
   if(snapstate == NULL) v5_error("can't allocate snapstate: anicon");

  dataload = false;
}

void AnimControl::start_anim(bool dl){

// printf("ani start 0\n");
 all_set = false;
 time_of_start = current_time;
 for(int i=0;i<data->ntime;i++)
 	dmade4anim[i] = 0;

 if(tracer->is_on()){
  tracer->integration_off();
  tracer->copy_seeds();
 }

 TimeOfLoadedData = -1;
 recover_flag = 0;
 ani_recalc = 0;
 printf("ani start\n");
 anim_on = true;
 dataload = dl;
}

void AnimControl::prepare_make_dlists(){
 //This function is for computation thread
 if(!anim_on) return; 

 int t = current_time;
 
 if(all_set && dataload) data->read_data(t);
 
 if(dmade4anim[t] == 1 
 	|| TimeOfLoadedData == t) return;

 data->read_data(t);

 if(volumerendering->is_on())
 	volumerendering->make_mindata(t);

 if(isosurf->is_on())
 	isosurf->make_mindata(t);
 
 if(tracer->is_on())
 	tracer->calc_all();

 TimeOfLoadedData = t;
 printf("time = %d is loaded\n", t);
};

void AnimControl::make_dlists_flagon(){
// this is for CAVE Master Display
// puts("mdflon start");
 if(!anim_on){
 return; 
 }
 
 int t = current_time;

 if(dmade4anim[t] == 1) return;

 if(t != TimeOfLoadedData) return;

 if(isosurf->is_on()){
  recalc->iso_flag_on();
 }
 
 for(int i=0;i<3;i++){
  if(orthoSlicer[i]->is_on())
	recalc->ortho_flag_on(i);
 }
 
 if(volumerendering->is_on()){
  recalc->vol_flag_on(1);
 }
 
 if(tracer->is_on()){
  recalc->tracer_flag_on();
 }
  ani_recalc = 1;
//  printf("time = %d ; recalcflag for anim\n", t);
}

void AnimControl::make_dlists4anim(){
// this is for Display Processes
 if(!anim_on) return;

 if(ani_recalc == 0) return;
 
 recalc->recalc_iso();
 for(int i=0;i<3;i++)
  recalc->recalc_ortho(i);

 recalc->recalc_tracer();
 recalc->recalc_volrend();

 CAVEDisplayBarrier();
 
 if(CAVEMasterDisplay()){
  int t = current_time;
  prev_frame = CAVEGetFrameNumber();
  prev_time  = CAVEGetTime();
  
  int next_t;
  next_t = t+1;
  if(next_t == data->ntime) next_t = 0;
  
  if(time_of_start == next_t) all_set = true;

  dmade4anim[t] = 1;
  ani_recalc = 0;
//  printf("time = %d ; makedlist for anim\n", t);
 }

 CAVEDisplayBarrier();
}

void AnimControl::recover1(){
 if(!anim_on){
 if(recover_flag == 1){
  int t = current_time;
  data->read_data(t);
  volumerendering->make_mindata(t);
  isosurf->make_mindata(t);

  recover_flag = 2;
 }
 return;
 }
}

void AnimControl::recover2(){
 if(!anim_on){
  if(recover_flag == 2){
	 if(tracer->is_on()){
  	  tracer->set_seeds();
  	 }

	 if(isosurf->is_on()){
	  recalc->iso_flag_on();
	 }
 
	 for(int i=0;i<3;i++){
	  if(orthoSlicer[i]->is_on())
		recalc->ortho_flag_on(i);
	 }

	 recalc->vol_flag_on(2);

	 recover_flag = 0;
//	printf("flag 2\n");
  }
 }
}

void AnimControl::forward_time(){
 if(!anim_on){
 return;
 }

 if(dmade4anim[current_time] == 0) return;
 // printf("Frame:%d - %d\n",CAVEGetFrameNumber(),prev_frame); 

 if(data->get_ldtime() != current_time && dataload) return;
 
 if(CAVEGetFrameNumber() - prev_frame < 10) return;
 if(all_set && (CAVEGetTime() - prev_time < intval)) return;
 if(!all_set && (CAVEGetTime() - prev_time < 0.5)) return;
 // forward time;
 int tctime = current_time;
 tctime++;
 if(tctime >= data->ntime) tctime = 0;
 current_time = tctime;
 prev_frame = CAVEGetFrameNumber();
 prev_time  = CAVEGetTime();
};

void AnimControl::stop(){
 if(!anim_on) return;
 anim_on = false;
 all_set = false;
 ani_recalc = 0;
 recover_flag = 1;
 dataload = false;
 printf("stop time = %d\n", current_time);
}

void AnimControl::OneStepForward(){
 int ttime = current_time;
 ttime++;
 if(ttime == data->ntime) current_time = 0;
 	else current_time = ttime;

 if(tracer->is_on()){
  tracer->integration_off();
  tracer->copy_seeds();
 }

 recover_flag = 1;
}


void AnimControl::OneStepBackward(){
 int ttime = current_time;
 ttime--;
 if(ttime == -1) current_time = data->ntime-1;
 	else current_time = ttime;

 if(tracer->is_on()){
  tracer->integration_off();
  tracer->copy_seeds();
 }

 recover_flag = 1;
}

///////////////////////////

void
AnimControl::make_vert_level_bar ()
{
  GLfloat diffuse[] = { 0.3, 0.3, 1.0, 1.0 };
  GLfloat ambient[] = { 0.0, 0.3, 0.3, 1.0 };
  GLfloat specular[] = { 0.3, 1.0, 0.3, 1.0 };

  glEnable (GL_LIGHTING);
  glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
  glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
  glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, specular);
  glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 10.0);

  float rad_top = vert_level_bar_rad (work_vmax);
  float rad_btm = vert_level_bar_rad (work_vmin);
  int tID = CAVEUniqueIndex();

  glPushMatrix ();
  if (work_vmax * work_vmin > 0.0)
    {				// Same sign. A cone.
      gluCylinder (cyl[tID], rad_btm, rad_top, vert_level_bar_height, 16, 2);
    }
  else
    {				// Change sign.
      float ratio_plus = work_vmax / work_dval;
      float ratio_minus = 1.0 - ratio_plus;
      float height_plus = vert_level_bar_height * ratio_plus;
      float height_minus = vert_level_bar_height * ratio_minus;
      gluCylinder (cyl[tID], rad_btm, 0.0, height_minus, 16, 2);
      glTranslatef (0.0, 0.0, height_minus);
      gluCylinder (cyl[tID], 0.0, rad_top, height_plus, 16, 2);
    }
  glPopMatrix ();
}

static void
arrow_color ()
{
  GLfloat diffuse[] = { 0.8, 0.8, 0.3, 1.0 };
  GLfloat ambient[] = { 0.3, 0.3, 0.0, 1.0 };
  GLfloat specular[] = { 1.0, 1.0, 0.3, 1.0 };

  glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
  glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
  glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, specular);
  glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 10.0);
}

void
AnimControl::left_arrow ()
  //    
  //     ||
  //     || /
  //     ||<  Make an left arrow which touches the vertical bar.
  //     || \
  //     ||
  //
{
  float rad = 0.1;
  float length = 0.5;
  glTranslatef (length + vert_level_bar_rad (work_level), 0.0, 0.0);
  glRotatef (-90.0, 0.0, 1.0, 0.0);
  gluCylinder (cyl[CAVEUniqueIndex()], rad, 0.0, length, 16, 2);
}

void
AnimControl::all_arrows ()
{
  int n_arrow = 8;
  float dtht = inl_r2d (TWOPI / n_arrow);
  glEnable (GL_LIGHTING);
  arrow_color ();
  for (int i = 0; i < n_arrow; i++)
    {
      glPushMatrix ();
      glRotatef (dtht * i, 0.0, 0.0, 1.0);
      left_arrow ();
      glPopMatrix ();
    }

}

void
AnimControl::wand_motion ()
{
  float pos[3];
  float dz;

  CAVEGetPosition (CAVE_WAND_NAV, pos);

  dz = pos[2] - snapstate->wand_pos_z ();

  float ratio = dz / vert_level_bar_height;
  float old_level = 1.0/intval;
//  float old_level = intval;
  float new_level = old_level + work_dval * ratio;
  if (new_level > work_vmax)
    new_level = work_vmax;

  if (new_level < work_vmin)
    new_level = work_vmin;

  work_level = new_level;
}

static float
rot_angle ()
{
  const float dphidt = 36.0;	// 1 rotation in 10 seconds.
  float t = CAVEGetTime ();
  return (dphidt * t);
}

void
AnimControl::draw_vertical_bar (float x, float y, float z,
				 float phi, float distance)
{
  //     
  //       ||
  //       || The radius of bar changes acoord to scalar value.
  //       ||
  //       ||
  //       ||
  //   --> || <--   level indicator by 8 arrows. They slides by wand.
  //       ||
  //       ||
  //       ||
  //

  x -= distance * sin (phi);	// Since phi=0 is y-axis.
  y += distance * cos (phi);
  z -= 1.5;			// Shift 0.5 feet down.

  float dlev = (work_level - work_vmin) / work_dval;
  float shift_z = dlev * vert_level_bar_height;

  glPushAttrib (GL_LIGHTING_BIT);
  glPushMatrix ();
  glTranslatef (x, y, z);
  make_vert_level_bar ();
  glPushMatrix ();
  glRotatef (rot_angle (), 0.0, 0.0, 1.0);	// Rotation around the bar.
  glTranslatef (0.0, 0.0, shift_z);
  all_arrows ();
  glPopMatrix ();
  glPopMatrix ();
  glPopAttrib ();
}


void
AnimControl::draw ()
{
  float scale;

  if (!anim_on)
    return;

  if(!CAVEBUTTON2) return;
  
  scale = scalefh->current_scale_is ();

  float x = snapstate->head_pos_x ();
  float y = snapstate->head_pos_y ();
  float z = snapstate->head_pos_z ();
  float distance = 3.0;		// feet away
  float phi = snapstate->head_phi () + inl_d2r (20.0);	// slightly left.

  glScalef (1.0 / scale, 1.0 / scale, 1.0 / scale);

  draw_vertical_bar (x, y, z, phi, distance);

  glScalef (scale, scale, scale);

}

float
AnimControl::vert_level_bar_rad (float level)
{
  return fabs (level) * vert_drad_dlevel;
}

void
AnimControl::button_has_pressed ()
{
  if(!anim_on || !all_set) return;
  snapstate->record_head_info ();	// When middle button is pressed.
  snapstate->record_wand_info ();	// Wand is used to shift the cut plane.
}

void
AnimControl::button_has_released ()
{
  if(!anim_on || !all_set) return;
//  intval = work_level;
  intval = 1.0/work_level;
}

void
AnimControl::button_being_pressed ()
{
  if(!anim_on || !all_set) return; 
  wand_motion ();
}

void AnimControl::writedown_ic(char name[], int n){
    FILE *fp;
    fp = fopen(name, "a");
    if (fp == NULL)
	v5_error("File open error: ic save. anicon");
    fprintf(fp, "TIME %d\n", current_time);
    fclose(fp);
}

