//
// vfive: Vector Field Interactive Visualization Environment
//
// data.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.17
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data* data;

extern Tracer*        tracer;
extern LocalArrows*   localArrows;
extern Isosurf*       isosurf;
extern OrthoSlicer*   orthoSlicer[3];
extern LocalSlicer*   localSlicer;
extern Snow*          snow;
extern Hotaru*        hotaru;
extern cave_volrend*  volumerendering;
extern Probe*      probe;
extern AnimControl *anicon;
extern Recalc *recalc;
extern CurveObj *curveObj;
extern SurfaceObj *surfaceObj;
extern Info_display *info;

Data::Data(char *inputFile_)
{
  int n;
  ntime = 1;
  skip4 = true;
    
  strcpy(inputFile, inputFile_);
  n = 0;
  while(inputFile[n] != '\0'){
   if(inputFile[n] == '\\') inputFile[n] = '/';
   n++;
  }

  v5_get_int(inputFile, "n1", &n1);  
  v5_get_int(inputFile, "n2", &n2);
  v5_get_int(inputFile, "n3", &n3);
  v5_get_int(inputFile, "nvec", &nvec);
  v5_get_int(inputFile, "nscal", &nscal);
  
  n = 0;
  v5_get_int(inputFile, "ntime", &n);
  if(n > 1) ntime = n;
  if(ntime > 1) td_flag = true;
  	else td_flag = false;  

  if(MAX_DLIST_TIME < ntime) v5_error("too mamy time steps!");
  
  n = 0;
  v5_get_int(inputFile, "NoSkip4", &n);
  if(n == 1) skip4 = false;
  
  v5_get_float(inputFile, "scale",   &scale);
  v5_get_float(inputFile, "shift_x", &shift_x);
  v5_get_float(inputFile, "shift_y", &shift_y);
  v5_get_float(inputFile, "shift_z", &shift_z);


  x = create_array_ff1(n1);     //  3D array 
  y = create_array_ff1(n2);     //  of field float.
  z = create_array_ff1(n3);

  //  Vector fields 
  for ( n = 0; n < nvec; n++) {
    // debug s
    printf("  vector n = %d\n", n);
    // debug e
    vector_fld[n] = new Field_v;
     if(vector_fld[n] == NULL) v5_error("can't allocate vector_fld: data");
    vector_fld[n]->x = create_array_ff3(n3, n2, n1); //  3D array 
    // debug s
    puts(" ----- x done.");
    // debug e
    vector_fld[n]->y = create_array_ff3(n3, n2, n1);
    // debug s
    puts(" ----- y done.");
    // debug e
    vector_fld[n]->z = create_array_ff3(n3, n2, n1);
    // debug s
    puts(" ----- z done.");
    // debug e
  }

  //  Scalar fields
  for (n = 0; n < nscal; n++) {
    // debug s
    printf("  scalar n = %d\n", n);
    // debug e
    scalar_fld[n] = new Field_s;
     if(scalar_fld[n] == NULL) v5_error("can't allocate scalar_fld: data");
    scalar_fld[n]->val = create_array_ff3(n3, n2, n1);
  }

  loaded_data_time = -1;
  ic_setting = false;
}

static void
read_coord(char compo) {
  FILE *fp;
  char dummy[4], key[50], coord_file[200];
   
  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

   sprintf(key, "%cfile", compo);
   v5_get_string(data->inputFile, key, coord_file);

   fp = fopen(coord_file,"rb");
   if (fp == NULL) v5_error(" Open failed. read_coordinates.");
   if(data->is_skip4()) fread(dummy, 1, 4, fp);

   if ( compo=='x' ){ 
     fread((char *)data->x, 1, n1*sizeof(ffloat_), fp);
    } else if ( compo=='y' ){
     fread((char *)data->y, 1, n2*sizeof(ffloat_), fp);
    } else if ( compo=='z' ){
     fread((char *)data->z, 1, n3*sizeof(ffloat_), fp);
    } else 
     v5_error(" Bad compo! read_coordinates.");
  
   fclose(fp);

}

static void
coord_trans()
{
  int i, j, k; 

  float norms = feet_per_meter * (data->scale);
  printf("  norms = %f\n", norms);
  
  //-- coordinate transform 
  for(i=0; i<data->n1; i++) data->x[i] += data->shift_x;
  for(j=0; j<data->n2; j++) data->y[j] += data->shift_y;
  for(k=0; k<data->n3; k++) data->z[k] += data->shift_z;

  //--- coordinate scaling so that unit 1 means 1 meter
  for(i=0; i<data->n1; i++) data->x[i] *= norms;
  for(j=0; j<data->n2; j++) data->y[j] *= norms;
  for(k=0; k<data->n3; k++) data->z[k] *= norms;
}


static void 
read_vect(int nthvec, char compo, int t)  
{
  FILE *fp;
  char dummy[4], vect_file[200], key[50];
    
  int n123 = (data->n1)*(data->n2)*(data->n3);
  
   // Only Master Node reads data file
   if(!data->is_td()) sprintf(key, "vect%d%c", nthvec, compo);
   	else  sprintf(key, "vect%d%ct%d", nthvec, compo, t);
	
   v5_get_string(data->inputFile, key, vect_file);
//   printf("v5: vect_file = %s\n", vect_file);
   fp = fopen(vect_file,"rb");
   if (fp == NULL) v5_error(" Open failed. read_vectors");

   if(data->is_skip4()) fread(dummy, 1, 4, fp);

  
   if ( compo=='x' ){
     fread((char *)(data->vector_fld[nthvec]->x[0][0]), 
	  1, n123*sizeof(ffloat_), fp);
   } else if ( compo=='y' ){
     fread((char *)(data->vector_fld[nthvec]->y[0][0]), 
	  1, n123*sizeof(ffloat_), fp);
   } else if ( compo=='z' ){
     fread((char *)(data->vector_fld[nthvec]->z[0][0]), 
	  1, n123*sizeof(ffloat_), fp);
   } else
     v5_error("*** Bad compo! read_vectors.");

   fclose(fp);
}


static void 
read_scalar(int nth, int t)  
{
  FILE *fp;
  char dummy[4], scal_file[200], key[50];

  if ( nth>data->nscal ) v5_error(" bad nth. read_scalars");
  
  int n123 = (data->n1)*(data->n2)*(data->n3);

  if(!data->is_td()) sprintf(key, "scal%d", nth);
	else sprintf(key, "scal%dt%d", nth, t);
	
  v5_get_string(data->inputFile, key, scal_file);
  fp = fopen(scal_file,"rb");
  if (fp==NULL) v5_error(" Open failed; read_scalars.");

  if(data->is_skip4()) fread(dummy, 1, 4, fp);

  fread((char *)(data->scalar_fld[nth]->val[0][0]), 
	1, n123*sizeof(ffloat_), fp);
  fclose(fp);

}


static void 
read()
{
  read_coord('x');
  read_coord('y');
  read_coord('z');

  for (int i=0; i<data->nvec; i++) {
    read_vect(i,'x', 0);
    read_vect(i,'y', 0);
    read_vect(i,'z', 0);
  }

  for (int n=0; n<data->nscal; n++)  
    read_scalar(n, 0);

  coord_trans();
}

static void
set_grid_size_min() 
{
  //  To find minimum grid size to estimate "dt" of the RK integration
  int   i,j,k;
  ffloat_ dxmin = 1.e20; /* something very large */
  ffloat_ dymin = 1.e20;
  ffloat_ dzmin = 1.e20;
  ffloat_ dx, dy, dz, dd;

  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

  for (i=0; i<n1-1; i++) {
    dx = data->x[i+1]-data->x[i];
    if (dx < 0) {
      v5_error("   x[i] must increase as i.");
      }
    if (dx < dxmin) dxmin = dx;
  }
  for (j=0; j<n2-1; j++) {
    dy = data->y[j+1]-data->y[j];
    if (dy < 0) {
      v5_error("   y[j] must increase as j.");
      }
    if (dy < dymin) dymin = dy;
  }
  for (k=0; k<n3-1; k++) {
    dz = data->z[k+1]-data->z[k];
    if (dz < 0) {
      v5_error("   z[k] must increase as k.");
      }
    if (dz < dzmin) dzmin = dz;
  }
  dd = dxmin < dymin ? dxmin : dymin;
  data->grid_size_min = (dd < dzmin ? dd : dzmin);
}
  

static void
coord_check()
{
  
  data->region_xmin = data->x[0];
  data->region_xmax = data->x[data->n1-1];
  data->region_ymin = data->y[0];
  data->region_ymax = data->y[data->n2-1];
  data->region_zmin = data->z[0];
  data->region_zmax = data->z[data->n3-1];

  printf("v5: coordinate xmin = %f\n", data->region_xmin);
  printf("v5: coordinate xmax = %f\n", data->region_xmax);
  printf("v5: coordinate ymin = %f\n", data->region_ymin);
  printf("v5: coordinate ymax = %f\n", data->region_ymax);
  printf("v5: coordinate zmin = %f\n", data->region_zmin);
  printf("v5: coordinate zmax = %f\n", data->region_zmax);

  if (data->region_xmin >= data->region_xmax || 
      data->region_ymin >= data->region_ymax || 
      data->region_zmin >= data->region_zmax)  {
    v5_error(" Something in coordinates is strnage.\n");
  }
}

static void
vect_check()
{    
  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

  for(int n=0; n<data->nvec; n++) {
    char temp1[200];
    char temp2[200];
    sprintf(temp1, "vectmin%d", n);
    sprintf(temp2, "vectmax%d", n);
    double min, max;
    int found1 = 0,found2 = 0;
    found1 = v5_get_double(data->inputFile, temp1,   &min);
    found2 = v5_get_double(data->inputFile, temp2,   &max);

    if(found1*found2>0 && data->is_td()){
    	data->vector_fld[n]->vmin = (ffloat_)min;
    	data->vector_fld[n]->vmax = (ffloat_)max;
	continue;
    }

    ffloat_ vmin =  1.e20;
    ffloat_ vmax = -1.e20;

    for (int i=0; i<n1; i++) {
      for (int j=0; j<n2; j++) {
	for (int k=0; k<n3; k++) {
	  ffloat_ vx = (data->vector_fld[n])->x[k][j][i];
	  ffloat_ vy = (data->vector_fld[n])->y[k][j][i];
	  ffloat_ vz = (data->vector_fld[n])->z[k][j][i];
	  ffloat_ vv = sqrt(vx*vx+vy*vy+vz*vz);

	  if(vv > vmax) vmax = vv;
	  if(vv < vmin) vmin = vv;

	}
      }
    }

    data->vector_fld[n]->vmin = vmin;
    data->vector_fld[n]->vmax = vmax;

    printf("v5: %d th vec: vmin = %f vmax = %f\n",n, vmin, vmax);
  }
}

static void 
scalar_check()  
{
  int i,j,k,n,ip,im,jp,jm,kp,km;
  ffloat_ vmin, vmax, vv;
  ffloat_ vol, dvol, avr, var;
  ffloat_ dx,dy,dz;

  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

  for(n=0; n<data->nscal; n++) {
    char temp1[200];
    char temp2[200];
    sprintf(temp1, "scalmin%d", n);
    sprintf(temp2, "scalmax%d", n);
    double min, max;
    int found1 = 0,found2 = 0;
    found1 = v5_get_double(data->inputFile, temp1,   &min);
    found2 = v5_get_double(data->inputFile, temp2,   &max);

    if(found1*found2>0 && data->is_td()){
    	data->scalar_fld[n]->vmin = (ffloat_)min;
    	data->scalar_fld[n]->vmax = (ffloat_)max;
	continue;
    }
    
    vmin =  1.e20;
    vmax = -1.e20;
    avr = 0.0;
    var = 0.0;
    vol = 0.0;
    for (i=0; i<n1; i++) {
      if     (i==0)    { ip = 1;    im = 0; }
      else if(i==n1-1) { ip = n1-1; im = n1-2; }
      else             { ip = i+1;  im = i-1; }
      dx = (data->x[ip] - data->x[im])*0.5;

      for (j=0; j<n2; j++) {
	if     (j==0)    { jp = 1;    jm = 0; }
	else if(j==n2-1) { jp = n2-1; jm = n2-2; }
	else             { jp = j+1;  jm = j-1; }
	dy = (data->y[jp] - data->y[jm])*0.5;

	for (k=0; k<n3; k++) {
	  if     (k==0)    { kp = 1;    km = 0; }
	  else if(k==n3-1) { kp = n3-1; km = n3-2; }
	  else             { kp = k+1;  km = k-1; }
	  dz = (data->z[kp] - data->z[km])*0.5;
	  
	  dvol = dx*dy*dz;
	  vv = data->scalar_fld[n]->val[k][j][i];

	  if(vv>vmax) vmax = vv;
	  if(vv<vmin) vmin = vv;

//		if(vv > 0.0 || vv < 0.0) printf("%f",vv);

	  vol += dvol;
	  avr += vv*dvol;
	  var += (vv*vv)*dvol;
	}
      }
    }
    avr /= vol;
    var /= vol;

    data->scalar_fld[n]->vmin = vmin;
    data->scalar_fld[n]->vmax = vmax;
    data->scalar_fld[n]->aver = avr;
    data->scalar_fld[n]->vari = sqrt(var - avr*avr);
    printf("v5:  Scalar field: vmin = %f\n", vmin);
    printf("v5:                vmax = %f\n", vmax);
    printf("v5:                aver = %f\n", avr);
    printf("v5:                vari = %f\n", sqrt(var - avr*avr));
  }
}

void
Data::set()
{
  read();
  loaded_data_time = 0;
  coord_check();
  vect_check();
  scalar_check();
  set_grid_size_min();
}

void
Data::read_data(int t)
{
  if(loaded_data_time == t) return;
  
  for (int i=0; i<nvec; i++) {
    read_vect(i,'x', t);
    read_vect(i,'y', t);
    read_vect(i,'z', t);
  }

  for (int n=0; n<nscal; n++)  
    read_scalar(n, t);

  loaded_data_time = t;
}


bool
Data::is_init_conditions(){
    char data_file[100];

    int found = v5_get_string(inputFile, "init_conditions_file", data_file);

    if(found == 0){
	ic_setting = false;
	return false;
    } else {
	ic_setting = true;
	return true;
     }
}

void 
Data:: set_init_conditions(){
  if(!is_init_conditions()) return;

  ic_setting = true;
  puts("init conditions exists!");
  char ini_file[100];

  v5_get_string(inputFile, "init_conditions_file", ini_file);

  int n = 0;
  while(ini_file[n] != '\0'){
   if(ini_file[n] == '\\') ini_file[n] = '/';
   n++;
  }

  int time = 0;
  if(!v5_get_int(ini_file, "TIME", &time)){
	time = 0;
  } else {
   anicon->set_ctime4init(time);
   anicon->recover1();
	while(anicon->get_recover_flag()){
		CAVEUSleep(1);
	}
  }
  printf("time = %d\n",time);
  // Navigation Matrix
    char ma[1024];
    if(v5_get_string_for_init(ini_file, "CAVENavMatrix", ma)){
	float matrix[4][4];
	char temp[25];
//        printf("%s\n",ma);
	sscanf(ma, "%s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f", temp, 
	&matrix[0][0], &matrix[0][1], &matrix[0][2], &matrix[0][3], 
	&matrix[1][0], &matrix[1][1], &matrix[1][2], &matrix[1][3], 
	&matrix[2][0], &matrix[2][1], &matrix[2][2], &matrix[2][3], 
	&matrix[3][0], &matrix[3][1], &matrix[3][2], &matrix[3][3]
	);
/*
	printf("%s\n",temp);
	 for(int j=0;j<4;j++){ 
	  for(int i=0;i<4;i++){
	   printf("%f ", matrix[j][i]);
	  }
	 }
	puts("matrix ends");
*/
	CAVENavLoadIdentity();
	CAVENavLoadMatrix(matrix);
    } else printf("No Navigation Matrix\n");

  int obj = 0;
  if(v5_get_int(ini_file, "CURVE", &obj)){
	curveObj->turn_on();
  }

  if(v5_get_int(ini_file, "SURFACE", &obj)){
	surfaceObj->turn_on();
  }

  if(v5_get_int(ini_file, "INFO", &obj)){
	info->turn_on();
  }

  //LocalArrows
    char la[1024];
    if(v5_get_string_for_init(ini_file, "LOCAL_ARROWS", la)){
	int la_vec;
	float pos[3];
	char temp[20];
	sscanf(la, "%s %d %f %f %f",temp, &la_vec, &pos[0], &pos[1], &pos[2]);
	localArrows->select_vec(la_vec);
	localArrows->set_pos(pos);
	localArrows->turn(1);

	printf("arrows  %d %f %f %f\n", la_vec, pos[0], pos[1], pos[2]);
    } else printf("No Local Arrows\n");

  //Isosurface
    char iso[1024];;
    if(v5_get_string_for_init(ini_file, "ISOSURF", iso)){
	int iso_scal;
	ffloat_ iso_value = 0.0;
	char temp[100];
	if(sizeof(ffloat_) == 4)
	sscanf(iso, "%s %d %f",temp,&iso_scal, &iso_value);
	else 
	sscanf(iso, "%s %d %lf",temp,&iso_scal, &iso_value);

	isosurf->select_scal(iso_scal);
	isosurf->set_level(iso_value);
	recalc->iso_flag_on();
	isosurf->turn(1);

	printf("ISOSURF: scal=%d value = %f\n",iso_scal,iso_value);
    } else printf("No ISOSOURF\n");

   //Snow
    char sw[1024];;
    if(v5_get_string_for_init(ini_file, "SNOW", sw)){
	int sw_vec;
	float sw_p[3], sw_u[3], sw_d[3], sw_r[3];
	char temp[100];
	sscanf(sw, "%s %d %f %f %f %f %f %f %f %f %f", temp,&sw_vec, &sw_p[0], &sw_p[1], &sw_p[2]
			, &sw_u[0], &sw_u[1], &sw_u[2], &sw_d[0], &sw_d[1], &sw_d[2]);

	printf("Snow %d %f %f %f %f %f %f %f %f %f\n",sw_vec, sw_p[0], sw_p[1], sw_p[2]
			,sw_u[0], sw_u[1], sw_u[2], sw_d[0], sw_d[1], sw_d[2]);

	vec_cross_normalf(sw_d, sw_u, sw_r);

	snow->select_vec(sw_vec);
	snow->set_pos(sw_p, sw_u, sw_d, sw_r);
	snow->turn(1);

    } else printf("No Snow\n");

  //Hotaru
    int htr = -1;
    if(v5_get_int(ini_file, "HOTARU", &htr)){
	hotaru->select_vec(htr);
	hotaru->place_hotaru();
	hotaru->turn(1);

      printf("Hotaru %d\n", htr);
    } else printf("No Hotaru\n");

   //OrthoSlicer I
    int osI_scal;
    int osI_num;
    char osI[1024];
    if(v5_get_string_for_init(ini_file, "ORTHO_SLICER_I", osI)){
	char temp[100];
	sscanf(osI, "%s %d %d", temp, &osI_scal, &osI_num);
	orthoSlicer[0]->select_scal(osI_scal);
	orthoSlicer[0]->set_plane(osI_num);
	recalc->ortho_flag_on(I);
	orthoSlicer[0]->turn_on();
	printf("Ortho Slicer I: scal=%d num = %d\n",osI_scal,osI_num);
    } else printf("No Ortho Slicer I\n");

   //OrthoSlicer J
    int osJ_scal;
    int osJ_num;
    char osJ[1024];
    if(v5_get_string_for_init(ini_file, "ORTHO_SLICER_J", osJ)){
	char temp[100];
	sscanf(osJ, "%s %d %d", temp, &osJ_scal, &osJ_num);
	orthoSlicer[1]->select_scal(osJ_scal);
	orthoSlicer[1]->set_plane(osJ_num);
	recalc->ortho_flag_on(J);
	orthoSlicer[1]->turn_on();
	printf("Ortho Slicer J: scal=%d num = %d\n",osJ_scal,osJ_num);
    } else printf("No Ortho Slicer J\n");

   //OrthoSlicer K
    int osK_scal;
    int osK_num;
    char osK[1024];
    if(v5_get_string_for_init(ini_file, "ORTHO_SLICER_K", osK)){
	char temp[100];
	sscanf(osK, "%s %d %d", temp, &osK_scal, &osK_num);
	orthoSlicer[2]->select_scal(osK_scal);
	orthoSlicer[2]->set_plane(osK_num);
	recalc->ortho_flag_on(K);
	orthoSlicer[2]->turn_on();
	printf("Ortho Slicer K: scal=%d num = %d\n",osK_scal,osK_num);
    } else printf("No Ortho Slicer K\n");

   //LocalSlicer
    char ls[1024];
    if(v5_get_string_for_init(ini_file, "LOCAL_SLICER", ls)){
	int ls_scal;
	float pos[3];
	char temp[100];
	sscanf(ls,"%s %d %f %f %f", temp, &ls_scal, &pos[0], &pos[1], &pos[2]);
	localSlicer->select_scal(ls_scal);
	localSlicer->set_pos(pos);
	localSlicer->turn_on();

	printf("local slicer:%d %f %f %f\n", ls_scal, pos[0], pos[1], pos[2]);
    } else printf("No Local Slicer\n");

   //directory
   char dir[256];
/*
   int num_char0 = 0;
   while(inputFile[num_char0] != '\0')
   {
	num_char0++;
   }
//    printf("initifle = %s:%d\n",ini_file, num_char);

   while(inputFile[num_char0] != '/')
   {
	num_char0--;
   }

   for(int i=0;i<num_char0;i++) dir[i] = inputFile[i];
*/
   int num_char = 0;
   while(ini_file[num_char] != '\0')
   {
	num_char++;
   }
//    printf("initifle = %s:%d\n",ini_file, num_char);

   while(ini_file[num_char] != '/')
   {
	num_char--;
   }
//    printf("initifle = %s:%d\n",ini_file, num_char);
//    dir[num_char0] = '/';
//   for(int i=0;i<num_char;i++) dir[i+num_char0+1] = ini_file[i];
   if(num_char == 0) dir[0] = '.';
    else for(int i=0;i<num_char;i++) dir[i] = ini_file[i];

    printf("dir = %s\n",dir);

   //Tracer
    for(int i=0;i<nvec; i++){
	char tfile[1024];
	int num_lines = 0;
	char the_key[100];
	sprintf(the_key, "TRACER%d", i);
	if(v5_get_string_for_init(ini_file, the_key, tfile)){
		char temp[100];
		char tr_posfile[100];
		sscanf(tfile, "%s %s %d", temp, tr_posfile, &num_lines);

		printf("TRACER FILE%d: %s\n", i, tr_posfile);
		printf("TRACER %d lines exist\n", num_lines);

		set_tracerdata(tr_posfile, dir, i, num_lines);

	} else {
		printf("No Tracer%d\n", i);
	}

    }

  //volume rendering
    int vr_scal;
    char vr[1024];;
    if(v5_get_string_for_init(ini_file, "VOLUME_RENDER", vr)){
	char temp[100];
	char vr_colfile[100];
	sscanf(vr, "%s %d %s", temp, &vr_scal, vr_colfile);
	printf("Volume Rendering: scal= %d datafile %s\n",vr_scal, vr_colfile);

	set_vrdata(vr_colfile, dir, vr_scal);

    } else printf("No Volume Rendering\n");

   //Probe
    for(int i=0;i<nvec; i++){
	char pfile[1024];
	char the_key[100];
	sprintf(the_key, "PROBEV%d", i);
	if(v5_get_string_for_init(ini_file, the_key, pfile)){
		char temp[100];
		float x,y,z;
		sscanf(pfile, "%s %f %f %f", temp, &x, &y, &z);

		printf("ProbeV %d %f %f %f\n", i, x,y,z);

	} else {
		printf("No probev%d\n", i);
	}

    }

    for(int i=0;i<nscal; i++){
	char pfile[1024];
	char the_key[100];
	sprintf(the_key, "PROBES%d", i);
	if(v5_get_string_for_init(ini_file, the_key, pfile)){
		char temp[100];
		float x,y,z;
		sscanf(pfile, "%s %f %f %f", temp, &x, &y, &z);

		printf("ProbeS %d %f %f %f\n", i, x,y,z);

	} else {
		printf("No probes%d\n", i);
	}

    }

    ic_setting = false;
}


void 
Data::set_tracerdata(char file[], char dir[], int v, int line_num){
    FILE *fp;
    char line[200];
    char dfile[300];
    sprintf(dfile, "%s/%s",dir,file);
    fp = fopen(dfile, "r");
    if (!fp)
	v5_error(" *** open failed. set_tracerdata");
    for(int i=0;i<line_num;i++){
	int direct, normed;
	float pos[3];
	fgets(line, 200, fp);
	sscanf(line, "%d %d %f %f %f", &direct, &normed, &pos[0], &pos[1],&pos[2]);

	tracer->select_vec(v);
	tracer->release(direct, normed, pos);
	tracer->turn(1);

	printf("d:%d n:%d %f %f %f\n", direct,normed, pos[0],pos[1],pos[2]);
     }
    fclose(fp);
  tracer->integration_on();
}

void 
Data::set_vrdata(char file[], char dir[], int s){
    FILE *fp;
    char line[200];
    unsigned char color[256][4];
    char dfile[300];
    sprintf(dfile, "%s/%s",dir,file);

    fp = fopen(dfile, "r");
    if (!fp)
	v5_error(" *** open failed. set_vrdata");

    for(int j=0;j<4;j++){
    for(int i=0;i<256;i++){
	int n, intensity;
	fgets(line, 200, fp);
	sscanf(line, "%d %d", &n, &intensity);
	if(intensity < 0) intensity = 0;
	if(intensity > 255) intensity = 255;
	color[i][j] = intensity;
//	printf("n:%d value:%d\n", n, intensity);
     }
     }
    fclose(fp);

    volumerendering->select_scal(s);
    volumerendering->set_coltable(s, color);
    volumerendering->remake_voldata(volumerendering->sscal());
    recalc->vol_flag_on(1);
    volumerendering->turn_on();
}
