//
// vfive: Vector Field Interactive Visualization Environment
//
// info_display.c++
//
//                by Nobuaki Ohno               2009.5.
//

#include "vfive.h"
extern Data *data;
extern Isosurf *isosurf;
extern OrthoSlicer *orthoSlicer[3];	// xy, yz, and zx planes.
extern AnimControl *anicon;
extern cave_message *cave_mssg;
extern LocalArrows *localArrows;
extern LocalSlicer *localSlicer;
extern Snow *snow;
extern Hotaru *hotaru;
extern cave_volrend *volumerendering;

//
// Displaying various information en face de the viewer
//
Info_display::Info_display(){
    on = 0;
    snapstate = new Snapstate;
    time_t jikan;
    snapstate->record_head_info();
    time(&jikan);
    sprintf(jikoku,"%s",ctime(&jikan));
}

void Info_display::update(){
    if(!on) return;
    time_t jikan;
    snapstate->record_head_info();
    time(&jikan);
    sprintf(jikoku,"%s",ctime(&jikan));
}

void Info_display::draw(void)
{
    if(!on) return;
    // In every frame snapstate gets head, wand positions and so on.
    float x = snapstate->head_pos_x();
    float y = snapstate->head_pos_y();
    float z = snapstate->head_pos_z();
    float distance = 4.0;	// feet away
    float phi = snapstate->head_phi();	// slightly left.

    char mssg[200];
    float kansan = data->scale * feet_per_meter;
    int i;

    x -= distance * sin(phi);	// Since phi=0 is y-axis.
    y += distance * cos(phi);
    z += 0.5;			// Shift 1.5 feet up.

    float vspace = 0.0;
    float space = 0.0;

    glPushAttrib(GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);


    glColor3f(1.0, 1.0, 1.0);

    glPushMatrix();

    glTranslatef(x, y, z);

    // Rotation to face the viewer
    glRotatef(phi * 180.0 / PI, 0.0, 0.0, 1.0);

    glTranslatef(-2.0, 0.0, 0.0);

    // Displaying date
    cave_mssg->write_message(0.25, jikoku, &space, &vspace);

    // Displaying the size of data
    if (data->ntime > 1) {

	sprintf(mssg, "SIZE = %d*%d*%d, TIME = %d [0-%d]\n",
		data->n1, data->n2, data->n3, 
		anicon->ctime(), data->ntime-1);

    } else {

	sprintf(mssg, "SIZE = %d*%d*%d\n", 
	 data->n1, data->n2, data->n3);

    }

    cave_mssg->write_message(0.25, mssg, &space, &vspace);


    if (localArrows->is_on()) {
	sprintf(mssg,
		"LocalArrows: No.%d\n",
		localArrows->svec());
	cave_mssg->write_message(0.25, mssg, &space, &vspace);
    }


    if (snow->is_on()) {
	sprintf(mssg, "SpotlightedSnow: No.%d\n", snow->svec());
	cave_mssg->write_message(0.25, mssg, &space, &vspace);
    }


    if (hotaru->is_on()) {
	sprintf(mssg, "Hotaru: No.%d\n", hotaru->svec());
	cave_mssg->write_message(0.25, mssg, &space, &vspace);
    }


    //----------------------------
    // SCALAR FIELD VISUALIZATION
    //----------------------------
    if (isosurf->is_on()) {
	sprintf(mssg,
		"Isosurface: No.%d, Lev = %.5f\n", isosurf->sscal(),
		isosurf->current_level_is());

	cave_mssg->write_message(0.25, mssg, &space, &vspace);
    }


    if (localSlicer->is_on()) {
	sprintf(mssg,
		"LocalSlicer: No.%d\n",
		localSlicer->sscal());
	cave_mssg->write_message(0.25, mssg, &space, &vspace);
    }



    for (i = 0; i < 3; i++) {

	if (orthoSlicer[i]->is_on()) {

	    switch (i) {

	    case I:
		sprintf(mssg,
			"OrthoSlicer Y-Z: No.%d, S-No.%d/%d\n",
			orthoSlicer[i]->sscal(),
			orthoSlicer[i]->get_plane(), data->n1-1);
		break;

	    case J:
		sprintf(mssg,
			"OrthoSlicer X-Z: No.%d, S-No.%d/%d\n",
			orthoSlicer[i]->sscal(),
			orthoSlicer[i]->get_plane(), data->n2-1);
		break;

	    case K:
		sprintf(mssg,
			"OrthoSlicer X-Y: No.%d, S-No.%d/%d\n",
			orthoSlicer[i]->sscal(),
			orthoSlicer[i]->get_plane(),data->n3-1);
		break;

	    }

	    cave_mssg->write_message(0.25, mssg, &space, &vspace);

	}


    }

    if (volumerendering->is_on()) {
	sprintf(mssg, "VolumeRendering: No.%d\n",
		volumerendering->sscal());
	cave_mssg->write_message(0.25, mssg, &space, &vspace);
    }

    glPopMatrix();
    glPopAttrib();

}
