//
// vfive: Vector Field Interactive Visualization Environment
//
//
// recalc.c++     
//
//           by Kageyama, Akira  (kage@nifs.ac.jp)    2000.04.20
//
//	Ohno 2009.5.14
//
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data *data;
extern Tracer*         tracer;
extern Isosurf*        isosurf;
extern OrthoSlicer*    orthoSlicer[3];    
extern cave_volrend*   volumerendering;
extern AnimControl*    anicon;

Recalc::Recalc(){
    for (int i = 0; i < CAVE_MAX_WALLS; i++) {
	for(int j=0;j<6;j++){
		recalc_flag[i][j] = 0;
	}
    }
}

void 
Recalc::iso_flag_on(){
  for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][0] = 1;
};

void 
Recalc::ortho_flag_on(int a){
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][a+1] = 1;
};

void 
Recalc::vol_flag_on(int n){
  for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][4] = n;
};

void 
Recalc::tracer_flag_on(){
  for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][5] = 1;
};

void
Recalc::recalc_volrend()
{
  int n;
  int tID = CAVEUniqueIndex();

  if (recalc_flag[tID][4] == 1) {
   n = volumerendering->sscal();

   if(!anicon->is_animon()) volumerendering->recalc(n);
	else volumerendering->ani_recalc();

   recalc_flag[tID][4] = 0;

  } else if(recalc_flag[tID][4] == 2){
  
  for(int i=0;i<data->nscal;i++)
   volumerendering->recalc(i);
   recalc_flag[tID][4] = 0;
  } 

}

void
Recalc::recalc_iso()
{
 int tID = CAVEUniqueIndex();
 if (recalc_flag[tID][0] == 1) {
    isosurf->calc();
    recalc_flag[tID][0] = 0;
  }
}

void
Recalc::recalc_ortho(int a)
{
  int tID = CAVEUniqueIndex();
  if (recalc_flag[tID][a+1] == 1) {
    orthoSlicer[a]->calc();
    recalc_flag[tID][a+1] = 0;
  }

}

void
Recalc::recalc_tracer()
{
 int tID = CAVEUniqueIndex();
 if (recalc_flag[tID][5] == 1) {
    tracer->calc();
    recalc_flag[tID][5] = 0;
  }
}

void 
Recalc::recalc_all(){
 if(!anicon->is_animon()) return;

 recalc_iso();

 for(int i=0;i<3;i++)
  recalc_ortho(i);

 recalc_tracer();
 recalc_volrend(); 

}

