//
// vfive: Vector Field Interactive Visualization Environment
//
//
// snapshot.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.04.24
//  		  Modified by Ohno (AVS file -> PPM file)
//
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Tracer*        tracer;
extern LocalArrows*   localArrows;
extern Isosurf*       isosurf;
extern OrthoSlicer*   orthoSlicer[3];
extern LocalSlicer*   localSlicer;
extern Snow*          snow;
extern Hotaru*        hotaru;
extern cave_volrend*  volumerendering;
extern Probe*      probe;
extern AnimControl *anicon;
extern Data *data;

extern CurveObj *curveObj;
extern SurfaceObj *surfaceObj;
extern Info_display *info;

Snapshot::Snapshot()
{
  active = 0;     // When this is 1, you cave save image to a file
                  // by pushing wand's middle button.
  counter = 0;
    time_t jikan;
    time(&jikan);
    sprintf(dir,"snapof%s",ctime(&jikan));
    int i=0;
    while(dir[i] != '\0'){
	if(dir[i] == ' ') dir[i] = '_';
	if(dir[i] == ':') dir[i] = '_';
	if(dir[i] == '\n') dir[i] = '\0';
	i++;
     }
    char command[128];
    sprintf(command,"mkdir %s", dir);
    system(command);
}


void Snapshot::initialize()
{
 if(CAVEMasterDisplay()){
  int org_x, org_y, width, height;
  CAVEGetWindowGeometry(&org_x, &org_y, &width, &height);

  pic_red = new unsigned char [width*height];
  pic_green = new unsigned char [width*height];
  pic_blue = new unsigned char [width*height];
  pic_rgb = new unsigned char [width*height*3];
 }
 
 CAVEDisplayBarrier();
 
}

/*
 *---------------------------------------------------------------------
 > makeAVSimage
 *---------------------------------------------------------------------
 * 
\*
     An OpenGL sample program.
	- Read the color buffer and make an AVS image file (*.x) from it 
	  by "glReadPixels" command.
	- An AVS image file named "picture.x" is automatically generated.
	- AVS image file has a simple data structure;
		    int  width
		    int  height
		    char alpha <-+  1st pixel
		    char red     |
		    char green   |   
		    char blue  <-+
		    char alpha <-+  2nd pixel
		    char red     |
		    char green   |   
		    char blue  <-+
		    char alpha <-+  3rd pixel
		    char red     |
		    char green   |   
		    char blue  <-+
		       .
		       .
		       .
		       .
                              by Akira Kageyama              <98.07.21>
				 Theory and Computer Simulation Center,
				 National Institute for Fusion Science.
*/


void 
Snapshot::makePPMimage(char *file, int org_x, int org_y, 
		       int width, int height) 
{
    FILE  *fp;

    // printf(" debug... width = %d height = %d\n", width, height);
    
    // We cannot use the following (simpler) command, since the
    // each element of RGBA in the color buffer isn't always the same as 
    // that in the AVS-X image file (which is "A-R-G-B").
    //
    // glReadPixels(org_x, org_y, width, height, 
    //   	    GL_RGBA, GL_UNSIGNED_BYTE, pic_rgba);

    // AVS file -> PPM file
    // Revised 2006.6 by N.Ohno

    glReadPixels(org_x, org_y, width, height, 
	         GL_RED,   GL_UNSIGNED_BYTE, pic_red);
    glReadPixels(org_x, org_y, width, height, 
                 GL_GREEN, GL_UNSIGNED_BYTE, pic_green);
    glReadPixels(org_x, org_y, width, height, 
                 GL_BLUE,  GL_UNSIGNED_BYTE, pic_blue);

    int ctr, i, j, anti_j, k;
    for (ctr = 0, j = 0; j < height; j++) {
	anti_j = height - j - 1;

	for (i = 0; i < width; i++, ctr += 3) {
	    k = anti_j * width + i;

	    pic_rgb[ctr + 0] = pic_red[k];
	    pic_rgb[ctr + 1] = pic_green[k];
	    pic_rgb[ctr + 2] = pic_blue[k];

	}
    }


    fp = fopen(file, "wb");
    if (fp == NULL)
	v5_error("File open error: image save.");
    fprintf(fp, "P6\n%d %d\n255\n", width, height);
    fwrite(pic_rgb, width * height * 3, 1, fp);

    fclose(fp);
}

static void
flush_white()
{
  glClearColor(1.0, 1.0, 1.0, 0.0);
  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
  glClearColor(0.0, 0.0, 0.0, 0.0);
}


void
Snapshot::save_image()
  //
  // Save image on the master screen into disk.
  //
{
  if ( CAVEMasterDisplay()==0 ) return;
	  // Do nothing when the child process is not for the master.

  if ( !active ) return;

  char file[100];
  sprintf(file,"%s/snapshot%03d.ppm",dir,counter);
  printf("  file = %s\n", file);
  int org_x, org_y, width, height;
  CAVEGetWindowGeometry(&org_x, &org_y, &width, &height);
  printf("saveimage: %s: %d x %d + %d + %d\n", 
	 file, width, height, org_x, org_y);
//  if ( width * height >= SNAPSHOT_IMAGE_MAX ) 
//    v5_error("The screen size is too large. Increase SNAPSHOT_IMAGE_MAX");
  glReadBuffer(GL_FRONT_RIGHT);
  makePPMimage(file, org_x, org_y, width, height);
  
  writedown_ics(counter);
  counter++;
  inactivate();
  flush_white();     // Notify the user that the picture was taken.
}

void
Snapshot::button_has_pressed()
{
  activate();
}

void 
Snapshot::writedown_ics(int counter){

 char ini_file[200];
 sprintf(ini_file,"%s/init_condition%d.v5",dir,counter);

 FILE *fp;
 fp = fopen(ini_file, "w");
 if (fp == NULL)
	v5_error("File open error: ic save. snapshot");
 fprintf(fp,"#VFIVE initial condition (%s)\n", data->inputFile);
 if(curveObj->is_on()) fprintf(fp, "CURVE 1\n");
 if(surfaceObj->is_on()) fprintf(fp, "SURFACE 1\n");
 if(info->is_on()) fprintf(fp, "INFO 1\n");
 float matrix[4][4];
 CAVENavGetMatrix(matrix);
 fprintf(fp, "CAVENavMatrix ");
 for(int j=0;j<4;j++){ 
  for(int i=0;i<4;i++){
   fprintf(fp, "%f ", matrix[j][i]);
  }
 }
 fprintf(fp,"\n");
 fclose(fp);

 anicon->writedown_ic(ini_file, counter);

 fp = fopen(ini_file, "a");
 if (fp == NULL)
	v5_error("File open error: ic save. head");
 float head_pos[3], head_up[3], head_front[3];

 CAVEGetPosition(CAVE_HEAD_NAV, head_pos);
 CAVEGetVector(CAVE_HEAD_UP_NAV, head_up);
 CAVEGetVector(CAVE_HEAD_FRONT_NAV, head_front);

 fprintf(fp,"HEAD %f %f %f %f %f %f %f %f %f\n", head_pos[0], head_pos[1], head_pos[2]
		, head_up[0], head_up[1], head_up[2], head_front[0], head_front[1], head_front[2]);
 fclose(fp);

 localArrows->writedown_ic(ini_file, counter);
 isosurf->writedown_ic(ini_file, counter);
 hotaru->writedown_ic(ini_file, counter);
 snow->writedown_ic(ini_file, counter);
 orthoSlicer[0]->writedown_ic(ini_file, counter);
 orthoSlicer[1]->writedown_ic(ini_file, counter);
 orthoSlicer[2]->writedown_ic(ini_file, counter);
 localSlicer->writedown_ic(ini_file, counter);
 tracer->writedown_ic(ini_file, dir, counter);
 volumerendering->writedown_ic(ini_file, dir, counter);
 printf("volren");
 probe->writedown_ic(ini_file, counter);
 printf("ic save ends");
}


void 
Snapshot:: removedir(void){

   if(counter != 0) return;

   char command[256];
   char rmcomm[100];

#ifdef WIN32
    sprintf(rmcomm,"rmdir /S /Q");
    sprintf(command,"%s %s",rmcomm, dir);
    system(command);
#else
    sprintf(rmcomm,"rm -r");
    sprintf(command,"%s %s/%s",rmcomm,tdir, dir);
    system(command);
#endif

}
