//
// vfive: Vector Field Interactive Visualization Environment
//
// tracer.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.17
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 


#include "vfive.h"

extern Data *data;
extern AnimControl *anicon;
extern GLUquadricObj *sphereObj[CAVE_MAX_WALLS];             // Ball

static
tracer_line_ptr**
create_array_2D_of_tracer_line_ptr(int ll, int mm)
{
    tracer_line_ptr **a;
    tracer_line_ptr *ptr;

    a = (tracer_line_ptr **)malloc(ll * sizeof(tracer_line_ptr *));
    if (!a) v5_error("create_array_2D_of_tracer_line_ptr: 1");

    ptr = (tracer_line_ptr *)malloc(ll * mm * sizeof(tracer_line_ptr));
    if (!ptr) v5_error("create_array_2D_of_tracer_line_ptr: 2");

    for (int i=0; i<ll; i++){
        a[i] = ptr +  i * mm;
    }

    return(a);

}


void
Tracer::set_nten(){
if(integration == 0) return;

  for (int v = 0; v < data->nvec; v++) {
    for (int i = 0; i < TRACER_LINE_MAX; i++) {
      if (line[v][i]->on == 0) continue;
      nten[v][i] = line[v][i]->nten;
    }
  }
}

Tracer::Tracer()
{
  int i;
  turn_off();
  integration_off();
  svec_ = 0;

  alin = create_array_i1(data->nvec);

  line = create_array_2D_of_tracer_line_ptr(data->nvec, TRACER_LINE_MAX);

  for (i=0; i<data->nvec; i++) {
    for (int j=0; j<TRACER_LINE_MAX; j++) {
      line[i][j] = new Tracer_line;
      line[i][j]->set_svec(i);
       if(line[i][j] == NULL) v5_error("can't allocate tracer_line: tracer");
	nten[i][j] = 0;
    }
  }

  beam = new PointerBeam(0.3, 1.0, 1.0, 3.0);
  if(beam == NULL) v5_error("can't allocate beam: tracer");

  for (int n=0; n<data->nvec; n++) 
    alin[n] = 0;

  Color wrk;
  i = 0;
  while (i < data->nvec) {
    vec_f2f(4, coltable[i++], wrk.mizuiro(0.0));
    if (i>=MAX_NVEC) break;
    vec_f2f(4, coltable[i++], wrk.aka(0.6));     // pink.
    if (i>=MAX_NVEC) break;
    vec_f2f(4, coltable[i++], wrk.midori(0.5));  // light green.
    if (i>=MAX_NVEC) break;
    vec_f2f(4, coltable[i++], wrk.murasaki(0.3));  
    if (i>=MAX_NVEC) break;
  }

  for (int v=0; v<data->nvec; v++) {
    for (int i=0; i<TRACER_LINE_MAX; i++) {
      	seeds[v][i][0] = 0.0;
      	seeds[v][i][1] = 0.0;
      	seeds[v][i][2] = 0.0;
	para[v][i] = 0;
    }
  }

}


static void
next_line(int& line)
{
  line = (++line) % TRACER_LINE_MAX;
}


void
Tracer::release()
{
  int sv = svec();

  switch (direct) {
  case POSI:
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(0);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, +1); // positive trace
    next_line(alin[sv]);
    break;

  case NEGA:
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(0);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, -1); // negative trace
    next_line(alin[sv]);
    break;

  case BOTH:
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(1);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, +1); // positive trace
    next_line(alin[sv]);
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(1);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, -1); // negative trace
    next_line(alin[sv]);
    break;
  }
}

void
Tracer::release(int dir, int normed, float p[3])
{
  int sv = svec();
  beam->tip[0] = p[0];  beam->tip[1] = p[1];  beam->tip[2] = p[2];

  switch (dir) {
  case POSI:
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(normed);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, +1); // positive trace
    next_line(alin[sv]);
    break;

  case NEGA:
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(normed);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, -1); // negative trace
    next_line(alin[sv]);
    break;

  case BOTH:
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(normed);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, +1); // positive trace
    next_line(alin[sv]);
    line[sv][alin[sv]]->erase();                  // delete the old line.
    line[sv][alin[sv]]->set_trace_normed_vec(normed);  // Trace raw vector.
    line[sv][alin[sv]]->start(beam, sv, -1); // negative trace
    next_line(alin[sv]);
    break;
  }
}

void
Tracer::copy_seeds()
{
 if(!on) return;
 
  for (int v=0; v<data->nvec; v++) {
    for (int i=0; i<TRACER_LINE_MAX; i++) {
	nten[v][i] = 0;
	para[v][i] = 0;
	normed[v][i] = 0;
      	seeds[v][i][0] = 0.0;
      	seeds[v][i][1] = 0.0;
      	seeds[v][i][2] = 0.0;
    }
  }

  for (int v=0; v<data->nvec; v++) {
    for (int i=0; i<TRACER_LINE_MAX; i++) {
      if (line[v][i]->on){
      	seeds[v][i][0] = line[v][i]->verts[0][0];
      	seeds[v][i][1] = line[v][i]->verts[0][1];
      	seeds[v][i][2] = line[v][i]->verts[0][2];
	para[v][i] = (char)line[v][i]->get_para();
	normed[v][i] = (char)line[v][i]->get_normed();
      }
	line[v][i]->erase();
    }
	alin[v] = 0;
  }

}

void Tracer::calc_all(){

  for (int v=0; v<data->nvec; v++) {
    for (int i=0; i<TRACER_LINE_MAX; i++) {
    	if(para[v][i] == 0) continue;
	line[v][i]->set_trace_normed_vec(1);
	line[v][i]->start(seeds[v][i], v, (int)para[v][i]);
    }
  }

  for (int v=0; v<data->nvec; v++) {
    for (int i=0; i<TRACER_LINE_MAX; i++) {
	if(!line[v][i]->on) continue;	
	for(int k=0;k<TRACER_NTEN_MAX;k++){
		if (line[v][i]->calc_state()) 
			line[v][i]->integrate();
	}
//	printf("nten %d\n",line[v][i]->nten);
    }
  }

}

void
Tracer::integrate()
{
  if (!integration) return;

//  int integ_l = 0;
  for (int v=0; v<data->nvec; v++) {
    for (int i=0; i<TRACER_LINE_MAX; i++) {
      if (line[v][i]->calc_state()){
	line[v][i]->integrate();
//        integ_l++;
      }
    }
  }
//  if(integ_l == 0) integration = 0;
}

void
Tracer::set_seeds()
{
  for (int v=0; v<data->nvec; v++) {
    for (int i=0; i<TRACER_LINE_MAX; i++) {
	nten[v][i] = 0;
//    	if(line[v][i]->on){
    	if(para[v][i]!=0){
	int n = alin[v];
	line[v][n]->erase();
	line[v][n]->set_trace_normed_vec((int)normed[v][i]);
	line[v][n]->start(seeds[v][n], v, (int)para[v][n]);
	next_line(alin[v]);
	}
    }
  }

  integration = 1;

}

void 
Tracer::draw_orbits(void)
{
  glLineWidth(1.0); // originary 2.0
  for (int v = 0; v < data->nvec; v++) {
    glColor3fv(coltable[v]);
    for (int i = 0; i < TRACER_LINE_MAX; i++) {
      if (line[v][i]->on == 0) continue;
      glPushMatrix();
      glBegin(GL_LINE_STRIP);
        // for (j=0; j<n; j+=20)    fast drawing by skip 
        for (int j=0; j < nten[v][i]; j++)
	  glVertex3fv(line[v][i]->verts[j]);
      glEnd();
      glPopMatrix();
    }
  }
}

void 
Tracer::calc(void)
{
  int tID = CAVEUniqueIndex();
  int ct = anicon->ctime();
  glNewList(dislis[tID][ct], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT | GL_LINE_BIT);
  glDisable(GL_LIGHTING);
  glPushMatrix();
  glLineWidth(1.0); // originary 2.0
  for (int v = 0; v < data->nvec; v++) {
    glColor3fv(coltable[v]);
    for (int i = 0; i < TRACER_LINE_MAX; i++) {
      if (line[v][i]->on == 0) continue;
      glBegin(GL_LINE_STRIP);
        for (int j=0; j < line[v][i]->nten; j++)
	  glVertex3fv(line[v][i]->verts[j]);
      glEnd();
    }
  }

  glPopMatrix();
  glPopAttrib();
  glEndList();
// printf("%d dlist success\n",ct);
}

void
Tracer::reset()
{
  int v = svec();
  alin[v] = 0;
  for (int i = 0; i < TRACER_LINE_MAX; i++) 
    line[v][i]->erase();
}

void 
Tracer::draw_particles()
{
  int tID;
  tID = CAVEUniqueIndex();

  for (int v=0; v < data->nvec; v++) {
    glMaterialfv(GL_FRONT, GL_DIFFUSE, coltable[v]);
    for (int i=0; i < TRACER_LINE_MAX; i++) {
      if (line[v][i]->on == 0) continue;
      int n = nten[v][i];
      float x = line[v][i]->verts[n][0];
      float y = line[v][i]->verts[n][1];
      float z = line[v][i]->verts[n][2];
      glPushMatrix();
        glTranslatef(x, y, z);   // tip of the orbit
        gluSphere(sphereObj[tID], PT_RADIUS, 8, 8);
      glPopMatrix();
    }
  }

}

void
Tracer::initialize() 
{
 int tID = CAVEUniqueIndex();
 for(int i=0;i<MAX_DLIST_TIME;i++)
  dislis[tID][i] = glGenLists(1);
}

void
Tracer::draw()
{
  if (!on) return;             // Now it's off.

  if(anicon->is_animon()){
   int tID = CAVEUniqueIndex();
   int ct = anicon->ctime();
   if(!anicon->is_completed(ct)) return;
   glCallList(dislis[tID][ct]);
   return;
  }
  
  glPushAttrib(GL_LIGHTING_BIT);
  glDisable(GL_LIGHTING); 
  beam->draw();
  draw_orbits();
  glEnable(GL_LIGHTING);
  draw_particles();
  glPopAttrib();
}

void
Tracer::button_being_pressed()
{
  if(!on) return;//2008.4.21
  beam->reset();
}

void
Tracer::button_has_pressed()
{
  beam->reset(); //2008.4.21
  beam->turn(1);
}

void
Tracer::button3_has_pressed()
{
  reset();
}


void
Tracer::button_has_released()
{
  beam->turn(0);
  release();      // Button release releases a ball!
}

void 
Tracer::writedown_ic(char name[], char dir[], int n){
    FILE *fp;
    for(int v=0;v<data->nvec;v++){
    int nl=0;
    for (int i=0; i < TRACER_LINE_MAX; i++)
      if (line[v][i]->on != 0) nl++;

    if(nl == 0) continue;

    char posname[50];
    fp = fopen(name, "a");
    if (fp == NULL)
	v5_error("File open error: ic save. tracer");
    sprintf(posname, "tr%d_posfile%d", v, n);
    fprintf(fp, "TRACER%d %s %d\n", svec_, posname, nl);
    fclose(fp);

    sprintf(posname, "%s/tr%d_posfile%d", dir, v, n);
    fp = fopen(posname, "w");
    if (fp == NULL)
	v5_error("File open error: ic save. tracer tr_posfile");
    for(int i=0;i<nl;i++){
    int direc, norm;
    if(line[v][i]->get_para() > 0) direc = 0;
	else direc = 1;
    norm = line[v][i]->get_normed();
    fprintf(fp, "%d %d %f %f %f\n",direc, norm, 
	line[v][i]->verts[0][0], line[v][i]->verts[0][1], line[v][i]->verts[0][2]);
     }
    fclose(fp);

     }
}




