#ifndef WIN32
#define GL_GLEXT_PROTOTYPES
#endif
#include <cave_ogl.h>
#include <GL/glu.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef WIN32
#include "glext.h"
#endif

#include "./words.h"

// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

const double  PI = 3.141592653589793;
const double  TWOPI = (2*PI);

const int  MENU_MAXROW   = 9;
const int  MENU_MAXCOL   = 7;
const int  PANEL_IMAGE_W = 128;
const int  PANEL_IMAGE_H = 64;
const int  MAX_NVEC      = 20;
const int  MAX_NSCAL     = 20;
const int  PANEL_NUM_MAX = 100;  
const int  PANEL_DEPTH_MAX = 20;

const int  TRACER_NTEN_MAX = 20000;   // Maximum tracing step

const int  TRACER_LINE_MAX = 200;
const int  LOC_ARROWS_POINT_MAX = 1000;

const int  SNOW_NUM = 5000;          // Snow particle number
const int  HOTARU_NUM = 10000;        // Number of 'hotaru's.

const int  I1 = 50;                   // For isosurface
const int  I2 = 50;
const int  I3 = 50;

const int  IV1 = 128;                   // For Volume Rendering
const int  IV2 = 128;
const int  IV3 = 128;

const float MAX_SCALE = 1.8;
const float MIN_SCALE = 0.2;


//===<Field float type>===
//      float or double.
//      Added on 2008.05.27. by A.K.
typedef float ffloat_;     
//typedef double ffloat_;  
//===</Field float type>===

const int MAX_DLIST_TIME = 10;

const char tdir[] = ".";
const char iso_dir[] = "iso";
const char slice_dir[] = "slice";
const char vr_dir[] = "volren";
const char tracer_dir[] = "tracer";

typedef  enum {I, J, K} axis_;        // Alias used in orthoslicer.
typedef  enum {POSI, NEGA, BOTH} trace_dir;    // Trace direction.
typedef  enum {NO,
	       TRACER,
	       LOCAL_ARROWS, 
	       ISOSURF,
	       SNOW,
	       HOTARU,
	       SNAPSHOT,
	       ORTHO_SLICER_I,
	       ORTHO_SLICER_J,
	       ORTHO_SLICER_K, 
	       LOCAL_SLICER,
	       VOLUME_RENDER,
	       PROBE,
	       SCALE_HANDLE,
	       ANIMATION
             } viz_kind;
//////////////////////////////////////////////////
//
// Runge Kutta Consts
// 2009.4
//////////////////////////////////////////////////

const  ffloat_  c01 = 1.0/9.0;
const  ffloat_  c02 = 0.4166666666666667e-1;
const  ffloat_  c03 = 0.125;
const  ffloat_  c04 = 1.0/6.0;
const  ffloat_  c05 = 0.5;
const  ffloat_  c06 = 0.6666666666666667;
const  ffloat_  c07 = 1.0/3.0;
const  ffloat_  c08 = 0.3750;
const  ffloat_  c09 = 0.1333333333333333e1;
const  ffloat_  c10 = 0.3333333333333333e1;
const  ffloat_  c11 = 0.7e1;
const  ffloat_  c12 = 0.9666666666666667e1;
const  ffloat_  c13 = 0.1533333333333333e2;
const  ffloat_  c14 = 0.6111111111111111;
const  ffloat_  c15 = 0.1166666666666667e1;
const  ffloat_  c16 = 0.1375e1;
const  ffloat_  c17 = 0.8333333333333333;
const  ffloat_  c18 = 0.4390243902439024;
const  ffloat_  c19 = 0.8780487804878049;
const  ffloat_  c20 = 0.1304878048780488e1;
const  ffloat_  c21 = 0.2097560975609756e1;
const  ffloat_  c22 = 0.2963414634146341e1;
const  ffloat_  c23 = 0.4317073170731707e1;
const  ffloat_  c24 = 0.3214285714285714e-1;
const  ffloat_  c25 = 0.4880952380952381e-1;
const  ffloat_  c26 = 0.2571428571428571;
const  ffloat_  c27 = 0.3238095238095238;

// radius of particle tracer
const float PT_RADIUS = 0.1;
//
//

static const float feet_per_meter = 3.280840;

//////////////////////////////////////////////////
// Class
//////////////////////////////////////////////////

class Color {
public:
  float*  aka(float f);
  float*  midori(float f);
  float*  ao(float f);
  float*  kiiro(float f);
  float*  murasaki(float f);
  float*  mizuiro(float f);
  float*  kuro(float f);
};

class PointerBeam {
  int    on;                // On (1) or off (0)
  float  length;            // length of the laser           
  float  beamcolor[3];      // RGB
  float  line_width;        // width of the beam
  void   drawLine(float *, float *);
public:
  PointerBeam(float, float, float, float);
  float  base[3];           // base point position
  float  tip[3];            // tip point position 
  float  direct[3];         // unit vector of wand direction
  void   reset(void);
  void   turn(int n) { n==1 ? on = 1 : on = 0; };
  void   draw(void);        // Draw the beam if it's on.
  int    is_on() {return on;}; 
};

class Target {
  viz_kind  viz_;
public:
  Target(void){viz_ = NO;};
  void      set(viz_kind v) { viz_ = v; };
  viz_kind  viz() { return(viz_); };
};

class Snapstate {
  float head_pos[3];        // head position
  float head_front[3];      // head direction
  float head_phi_;          // angle (in radian): NOTE! phi=0 is y-axis.
  float time_sec_;          // seconds from the program stated
  float wand_pos[3];        // wand position in simulation coord.
public:
  Snapstate(void);
  void  record_head_info();
  void  record_wand_info();
  void  record_time() { time_sec_ = CAVEGetTime(); };
  float head_phi()    { return head_phi_; };
  float head_pos_x()  { return head_pos[0]; };
  float head_pos_y()  { return head_pos[1]; };
  float head_pos_z()  { return head_pos[2]; };
  float wand_pos_x()  { return wand_pos[0]; };
  float wand_pos_y()  { return wand_pos[1]; };
  float wand_pos_z()  { return wand_pos[2]; };
};

typedef void(*Panf)(void);  // Panel function (ptr of a func that returns int).

class Panel {
  int    nrow;             // number of rows (title strings)
  char   string[10][50];
  Panf   exec_fp[10];      // exec this func while beem is shooting.
  Panf   exec_gp[10];      // exec this func when beem shoots in exit time.
  float  pos[10][3];       // position of each panel's center.
  int    active_row;
  int    selected_row;
  GLuint activeBorder_dislis[CAVE_MAX_WALLS][10];
  GLuint inactiveBorder_dislis[CAVE_MAX_WALLS][10];
  GLuint selectedBorder_dislis[CAVE_MAX_WALLS][10];
  GLuint image_texObj[CAVE_MAX_WALLS][10];
  void   make_border(int);
  void   make_activeBorder(int, GLuint);
  void   make_inactiveBorder(int, GLuint);
  void   make_selectedBorder(int, GLuint);
  int    is_it_shot(PointerBeam *, float *);
public:
  int    level;            // 0 (main), 1 (sub), 2(sub-sub), ...
  void   make(int, char*,      Panf=NULL, Panf=NULL, 
	           char* = "", Panf=NULL, Panf=NULL,
  	           char* = "", Panf=NULL, Panf=NULL, 
	           char* = "", Panf=NULL, Panf=NULL,
	           char* = "", Panf=NULL, Panf=NULL,
	           char* = "", Panf=NULL, Panf=NULL,
	           char* = "", Panf=NULL, Panf=NULL, 
	           char* = "", Panf=NULL, Panf=NULL,
	           char* = "", Panf=NULL, Panf=NULL, 
	           char* = "", Panf=NULL, Panf=NULL);
  void   make2(int, int, char *, Panf);
  void   make_display_list();
  void   make_image(char *, unsigned long, unsigned long, char *, char *);
  void   set_image(int, char*);
  void   show();
  void   check_and_exec_f(PointerBeam *);
  void   check_and_exec_g();
  void   exec_g(PointerBeam *);
  int    srow() { return(selected_row); };
  int    arow() { return(active_row); };
};


class Menu {                // Main menu, using Panel objects
  void   name_the_panels();
  void   set_panels();
  void   check();
  Snapstate *snapstate;      // Save states when left button is pressed.
  int    depth;             // Menu off when depth < 0.
public:  
  Menu();
  Panel* panel_at[PANEL_DEPTH_MAX];
  PointerBeam *beam;
  int    panel_num;
  Panel* panels[PANEL_NUM_MAX];
  float  appear_angle_phi, dphi;
  float  distance, panel_w, panel_h, top_pos;
  float  image_w, image_h;
  float  shoot_ball_rad;
  void   make_panel_image(char *);
  void   delete_panel_image(void);
  void   initialize();
  void   show();
  void   button_being_pressed();
  void   button_has_pressed();
  void   button_has_released();
  float  snap_head_pos_x() { return snapstate->head_pos_x(); };
  float  snap_head_pos_y() { return snapstate->head_pos_y(); };
  float  snap_head_pos_z() { return snapstate->head_pos_z(); };
  int    on() { return depth >= 0 ? 1 : 0; };
  void   set_depth(int i) { depth = i; };
  int running;
};


struct Field_v {
  ffloat_    ***x;          // x-component; x[n3][n2][n1]       
  ffloat_    ***y;          // y-           y[n3][n2][n1]       
  ffloat_    ***z;          // z-           z[n3][n2][n1]       
  ffloat_    vmin, vmax;
};


struct Field_s {
  ffloat_    ***val;        // Will be accessed as val[N3][N2][N1]
  ffloat_    vmin, vmax, aver, vari;
};


class Data {
  bool skip4;
  bool td_flag;
  int  loaded_data_time;
  bool ic_setting;
public:
  Data(char *);
  char      inputFile[200];
  ffloat_   *x;          // x-coordinate, will be accessed as x[n1]
  ffloat_   *y;          // y-          , will be accessed as y[n2]
  ffloat_   *z;          // z-          , will be accessed as z[n3]
  Field_v   *vector_fld[MAX_NVEC];
  Field_s   *scalar_fld[MAX_NSCAL];
  ffloat_   grid_size_min; 
  ffloat_   region_xmax, region_xmin,   // data defined range
            region_ymax, region_ymin,   
            region_zmax, region_zmin;
  int       n1, n2, n3;  // grid size
  int       nvec;        // # of vector fields
  int       nscal;       // # of scalar fields
  int	      ntime;	 //
  float     scale;       // scale parameter. user specified.
  float     shift_x, shift_y, shift_z;
  void      set();
  void	read_data(int t);
  bool	is_skip4(){return skip4;}
  bool	is_td(){return td_flag;}
  int       get_ldtime(){return loaded_data_time;};
  bool is_init_conditions();
  void set_init_conditions();
  void set_tracerdata(char *, char*,int, int);
  void set_vrdata(char *, char*,int);
  bool is_icsetting(){ return ic_setting;}
};

class  Tracer_line {
  ffloat_   h;                    // Integration step
  int       trace_normed_vec;      // Yes (1) or no (0)
  int       svec;
public:
  Tracer_line();
  int       on;                         // On (1) or off (0)
  float     verts[TRACER_NTEN_MAX][3];  // in x,y,z coordinate (not u1,2,3) 
  int       nten;                       // Vertex number
  int       calcToggle;                 // Calculation of the orbit on/off  
  ffloat_   ballPos[3];                 // Ball position at the curve's tip
  int       genRkh(ffloat_ *, ffloat_ *y, int, ffloat_ *,
		   int , int (*func)(ffloat_, ffloat_ *, ffloat_ *, int));
  void      calc_stop() {calcToggle = 0;};
  void      start(PointerBeam *, int, int);
  void      start(float start[3], int, int);
  void      erase() {on=0; nten=0;};
  int       calc_state() {return(calcToggle);};
  void      integrate();
  void      set_trace_normed_vec(int i) { trace_normed_vec = i; };
  void      set_svec(int n){svec = n;};
  int	    get_para(){if(h>0) return 1; else return -1;}
  int	    get_normed(){return trace_normed_vec;}
};

typedef Tracer_line *tracer_line_ptr;

class Tracer {
  int       on;                         // Appears (1) or not (0)
  int       integration;
  int       svec_;                      // selected_vec;
  int       *alin;                      // alin[NVEC]. Active line
  trace_dir direct;                     // POSI or NEGA or BOTH
  float     coltable[MAX_NVEC][4];	// corrected by ohno 2008.3.12
  tracer_line_ptr  **line;    // tracer_line[NVEC][TRACER_LINE_MAX]; 
  short     nten[MAX_NVEC][TRACER_LINE_MAX];
  void      draw_orbits();
  void      draw_particles();
  PointerBeam* beam;
  void      release();
  GLuint    dislis[CAVE_MAX_WALLS][MAX_DLIST_TIME];
  float	    seeds[MAX_NVEC][TRACER_LINE_MAX][3];
  char	    para[MAX_NVEC][TRACER_LINE_MAX];
  char      normed[MAX_NVEC][TRACER_LINE_MAX];
public:
  Tracer();
  void	    set_nten();
  void      integration_on() { integration=1; };
  void      integration_off() { integration=0; };
  void      turn_on(trace_dir d) { on=1; direct=d;};
  void      turn_off() { on=0; };
  void      reset();
  void      initialize();
  void      draw();
  void      integrate();
  void      select_vec(int i) {svec_ = i;};
  int       svec() {return(svec_);};
  void      button_being_pressed();
  void      button_has_pressed();
  void      button_has_released();
  void      button3_has_pressed();
  void      copy_seeds();
  void      calc_all();
  void      calc();
  void      set_seeds();
  int	    is_on(){return on;};

  void release(int, int, float*);
  void turn(int i){on=i;}
  void writedown_ic(char [], char [], int);
};

class Snow {
  int       on;                         // Appears (1) or not (0)
  int       integration;
  int       svec_;                      // selected_vec;
  float     snow_pos[SNOW_NUM][3];      // x,y,z
  float     snow_col[SNOW_NUM][3];      // x,y,z
  float     spot_distance;
  float     spot_angle;
  void      reposit_snow(int);
  float     wand_pos[3];     // These four vectors are used 
  float     wand_dir[3];     // to define the spot light cone.
  float     wand_rit[3];  
  float     wand_up[3];
  int       refresh_spot;
  int       is_in_spot_light(float*);
  float     weighted_random_rsq();
  float     weighted_random_sint();
  void      set_spot_light();
public:
  Snow(float, float);
  void      turn(int);
  void      initialize();
  void      draw();
  void      integrate();
  void      select_vec(int i) {svec_ = i;};
  int       svec() {return(svec_);};
  void      button_being_pressed();
  void      button_has_pressed();
  void      button3_has_pressed();
  void      button_has_released();
  int	    is_on(){return on;};

  void set_pos(float p[3], float u[3], float d[3], float r[3]);
  void writedown_ic(char [], int);
};

class Hotaru {
  int       on;                         // Appears (1) or not (0)
  int       svec_;                      // selected_vec;
  float     hotaru_pos[HOTARU_NUM][3];      // x,y,z
  int       is_in_the_region(float*);
  float     xmax, xmin, ymax, ymin, zmax, zmin, xlen, ylen, zlen;
  void      reposit_hotaru_i(int);
public:
  Hotaru();
  void      turn(int i) { i==1 ? on=1 : on=0; };
  void      place_hotaru();
  void      initialize();
  void      draw();
  void      integrate();
  void      select_vec(int i) {svec_ = i;};
  int       svec() {return svec_;};
  void      button3_has_pressed();
  int	    is_on(){return on;};
  void writedown_ic(char [], int);
};


class LocalArrows {
  int       on;                         // On (1) or off (0)
  int       svec_;                      // selected vector;
  PointerBeam* beam;
  ffloat_    dx[LOC_ARROWS_POINT_MAX];
  ffloat_    dy[LOC_ARROWS_POINT_MAX];
  ffloat_    dz[LOC_ARROWS_POINT_MAX];
  int       npoint;
  GLuint    mat0_dislis[CAVE_MAX_WALLS], mat1_dislis[CAVE_MAX_WALLS];
  void      material();
  int       position_set;             // recalc beam position (1) or not (0)
public:
  LocalArrows();
  void      initialize();
  void      draw();
  void      calc();
  void      turn_on() { on = 1; beam->turn(1); };
  void      turn_off() { on = 0; beam->turn(0); };
  void      select_vec(int i) { svec_ = i; };
  int       svec() {return svec_;};
  void      button_being_pressed();
  void      button_has_pressed();
  void      button_has_released();
  void      button3_has_pressed();
  int	     is_on(){return on;};

  void turn(int i){on=i;}
  void set_pos(float *);
  void writedown_ic(char [], int);
};


class CurveObj {
  char      *inputFile;
  int       on;                      // On (1) or off (0) 
  GLuint    dislis[CAVE_MAX_WALLS];
  int       nhon;
  int       ntenmax;
  char      xfile[100], yfile[100], zfile[100], cfile[100];
  float     **x;                     // x[nhon][nten] 
  float     **y;                     // y[nhon][nten] 
  float     **z;                     // z[nhon][nten] 
  float     **c;                     // c[nhon][nten] 
  int       *nten;                   // nten[nhon]    
public:
  void      initialize();
  void      turn(int n) { n == 1 ? on = 1 : on = 0; };
  void      make(char *);
  void      read();
  void      draw();
  void      toggle_on_off() { on == 1 ? on = 0 : on = 1; };
  void turn_on (){on = 1;};
  int is_on() {return on;}
};


class SurfaceObj {
  char      *inputFile;
  int       on;                      // On (1) or off (0) 
  GLuint    dislis[CAVE_MAX_WALLS];
  char      xfile[100], yfile[100], zfile[100];
  int       ni_max, nj_max, nmen;
  int       *ni, *nj;                // ni[nmen], nj[nmen]
  float     ***x;                    // x[nmen][nj_max][ni_max]
  float     ***y;                    // y[    ][      ][      ]    
  float     ***z;                    // z[    ][      ][      ]
  float     ***nx;                   // nx[nmen][nj_max][ni_max]
  float     ***ny;                   // ny[    ][      ][      ]    
  float     ***nz;                   // nz[    ][      ][      ]    
  int       find_ni(int);            // To find mesh size in i-direct
  int       find_nj(int);
  int       check_topology_in_i(int);
  int       check_topology_in_j(int);
  char      check_topology(int);
  void      construct_each_surface(int);
  void      make_normal(int, char);
  void      triangle_strip(int);
public:
  void      initialize();
  void      turn(int n) { n==1 ? on = 1 : on = 0; };
  void      make(char *);
  void      read();
  void      draw();
  void      toggle_on_off() { on == 1 ? on = 0 : on = 1; };
  void turn_on (){on = 1;};
  int is_on() {return on;}
};


class OrthoSlicerHandle {
  int       on;
  GLuint    dislis[CAVE_MAX_WALLS];
  axis_     axis;
  int       cut_plane;
  void      plane_xy(float);
  void      plane_yz(float);
  void      plane_zx(float);
  Snapstate *snapstate;    // Save states when middle button is pressed.
  float     scale;        // Whole region appears with this down scale factor.
public:
  OrthoSlicerHandle(axis_, int);
  void      turn(int n) { n==1 ? on = 1 : on = 0; };
  void      initialize(axis_, int);
  void      wand_motion(int);
  void      draw();
  void      snap();
  void      initialize();
  int       cut_plane_is() { return cut_plane; };
};

class OrthoSlicer {
  int       on;                      // On (1) or off (0) 
  int       sscal_;                  // Selected scalar field.
  axis_     axis;                    //
  int       plane;
  GLuint    dislis[CAVE_MAX_WALLS][MAX_DLIST_TIME];
  float     vmax, vmin;
  void      set_color(ffloat_, float *);
  void      find_maxmin_i(int, int);
  void      ortho_slicer_i(int);
  void      find_maxmin_j(int, int);
  void      ortho_slicer_j(int);
  void      find_maxmin_k(int, int);
  void      ortho_slicer_k(int);
  OrthoSlicerHandle* handle;
public:
  OrthoSlicer(axis_, int);
  void      turn_on() { on=1; };
  void      turn_off() { on=0; };
  void      draw();
  void      initialize();
  void      select_scal(int i) {sscal_ = i;};
  int       sscal() {return(sscal_);};
  void      calc();
  void      button_has_pressed();
  void      button3_has_pressed();
  void      button_being_pressed();
  void      button_has_released();
  int       get_plane(){return plane;}
  int	    is_on(){return on;};

  void set_plane(int p){plane=p;}
  void writedown_ic(char [], int);
};

class LocalSlicer {
  int       on;                      // On (1) or off (0) 
  int       sscal_;                  // Selected scalar field.
  int       svec_;                   // Selected vector field
  int       ndiv;                    //  a panel of ndiv x ndiv mesh 
  float     side;                    // feet 
  PointerBeam *beam;
  void      set_color(ffloat_, float *, ffloat_, ffloat_);
  void      check_vmaxmin(float*, float*, float*, ffloat_*, ffloat_*);
  void      slice(float *, float *);
public:
  LocalSlicer(int, float);
  void      initialize();
  void      turn_on() { on=1; };
  void      turn_off() { on=0; };
  void      draw();
  void      calc();
  void      select_vec(int i) {svec_ = i;};
  void      select_scal(int i) {sscal_ = i;};
  int       sscal() {return(sscal_);};
  int       svec() {return(svec_);};
  void      button_being_pressed();
  void      button_has_pressed();
  void      button_has_released();
  void      button3_has_pressed();
  int	    is_on(){return on;};

  void set_pos(float *);
  void writedown_ic(char [], int);
};

class BoundaryBox {
  int       on;
  GLuint    dislis[CAVE_MAX_WALLS];
public:
  BoundaryBox ();
  void      initialize();
  void      turn(int n) { n==1 ? on = 1 : on = 0; };
  void      draw();
  int       dislis_no(){return dislis[CAVEUniqueIndex()];};
};

class Snapshot {
  int       active;
  unsigned char      *pic_red;//[SNAPSHOT_IMAGE_MAX];  
  unsigned char      *pic_green;//[SNAPSHOT_IMAGE_MAX];
  unsigned char      *pic_blue;//[SNAPSHOT_IMAGE_MAX];
  unsigned char      *pic_rgb;//[SNAPSHOT_IMAGE_MAX*3];
  void      makePPMimage(char*, int, int, int, int);
  char dir[256];
  int counter;
public:
  Snapshot();
  void      activate() { active = 1; };
  void      inactivate() { active = 0; };
  void      button_has_pressed();
  void      save_image();
  void      initialize();//2009.5.13
  void      writedown_ics(int);
  void      removedir();
};


class IsosurfHandle {
  int       on;
  Snapstate *snapstate;    // Save states when middle button is pressed.
  float     vert_level_bar_height, vert_level_bar_rad_max;
  float     vert_level_bar_rad(float);
  void      left_arrow();
  void      all_arrows();
  void      make_vert_level_bar();
  void      make_level_rings();
  void      ring(float);
  float     work_vmax, work_vmin, work_dval, work_aver, work_vari;
  float     work_level;   // The arrows indicates this level while
                          // the handle is on. When wand's button 
                          // is released, this level is sent to isosurf.
  float     vert_drad_dlevel;  // bar radius change accord to the level.
public:
  IsosurfHandle();
  void      turn(int n) { n==1 ? on = 1 : on = 0; };
  void      wand_motion();
  void      draw_vertical_bar(float, float, float, float, float);
  void      draw();
  void      snap();
  void      button_has_pressed();
  void      button_has_released();  
};


class Isosurf {              
  int       on;                      // On (1) or off (0) 
  int	    *mindata;
  ffloat_   x[I3][I2][I1];
  ffloat_   y[I3][I2][I1];
  ffloat_   z[I3][I2][I1];
  ffloat_   ***f[MAX_NSCAL];           // Called as f[nscal][I3][I2][I1].
  float     color[MAX_NSCAL][4];     // RGB color of the surface
  int       sscal_;                  // selected scalar field
  ffloat_   level[MAX_NSCAL];        // Isosurface level
  GLuint    dislis[CAVE_MAX_WALLS][MAX_DLIST_TIME];       // display list 
  unsigned  char cell_code(int, int, int);
  void      vert_int_position(int, int *, int *, int *);
  ffloat_   c1x[I3][I2][I1], c1y[I3][I2][I1], c1z[I3][I2][I1];
  ffloat_   c2x[I3][I2][I1], c2y[I3][I2][I1], c2z[I3][I2][I1];
  ffloat_   c3x[I3][I2][I1], c3y[I3][I2][I1], c3z[I3][I2][I1]; //2008.4.9
  void      gencoord_prep(ffloat_ c1x[I3][I2][I1], 
			  ffloat_ c1y[I3][I2][I1],
			  ffloat_ c1z[I3][I2][I1],
			  ffloat_ c2x[I3][I2][I1],
			  ffloat_ c2y[I3][I2][I1],
			  ffloat_ c2z[I3][I2][I1],
			  ffloat_ c3x[I3][I2][I1],
			  ffloat_ c3y[I3][I2][I1],
			  ffloat_ c3z[I3][I2][I1]);
  void      grad(int, int, int, float *, float *, float *);
  void      cross_point(int, float *, float *);
  void      add_triangle_geom(float *, float *, float *, 
			      float *, float *, float *);
  void      init_coord();
  void      init_data();
  void      init_level();
  void      unit_cube();
  IsosurfHandle* handle;
  void save_mindata(int, int);
public:
  Isosurf();
  void      initialize();
  void      turn(int n) { n==1 ? on = 1 : on = 0; };
  int       sscal() {return sscal_;};
  void      draw();
  void      calc();
  void      button_being_pressed();
  void      button_has_pressed();
  void      button_has_released();
  void      button3_has_pressed();
  void      select_scal(int i) { sscal_ = i; };
  float     current_level_is() { return level[sscal_]; };
  void      set_level(ffloat_ lev) { level[sscal_] = lev; };
  int	    is_on(){return on;};
  void	    load_mindata(int t);
  void	    make_mindata(int t);
  void writedown_ic(char [], int);
};

class Probe {

  PointerBeam* beam;
  Snapstate *snapstate;
  int on;
  int v_on[MAX_NVEC], s_on[MAX_NSCAL];
  int chosen_var;
  ffloat_ sval[MAX_NSCAL],    vval[MAX_NVEC][4];
  int     s_outaregion[MAX_NSCAL], v_outaregion[MAX_NVEC];
  ffloat_ spos[MAX_NSCAL][3], vpos[MAX_NVEC][3];
  char scal_name[MAX_NSCAL][50], vec_name[MAX_NVEC][50];
  public:
  Probe();
  void probe_calc();
  void button_being_pressed();
  void button_has_pressed();
  void button3_has_pressed();
  void button_has_released();

  void vec_turn_on(int n){chosen_var = n; on = 1;};
  void scal_turn_on(int n){chosen_var = n + MAX_NVEC; on = 1;};
  void turn_off(){on = 0;}

  void draw();
  int  is_on(){return on;};
  void writedown_ic(char [], int);
};

/*******************************************************************/
/* Volume Rendering */
/*******************************************************************/

struct slicevertex {
    float vertx[6][3];
};

struct slicedata {
    int num_slice;
    int slice_max_limit;
    int *angles;
    float delta;
    struct slicevertex *slicevertx;
};

struct coloreditordata {
    int element;
    int prev_h, prev_v;
    int grab_f;
    int hsphere, vsphere;
    int ray_exist;
};


class cave_volrend {

    int on;
    int *mindata;
    unsigned char    ****volumedata;   // Called as f[nscal][IV3][IV2][IV1].
    unsigned char texvoldata[IV3][IV2][IV1*4];

    unsigned char colortable[MAX_NSCAL][256][4];
    GLuint tex[CAVE_MAX_WALLS][MAX_NSCAL];
    GLuint anitex[CAVE_MAX_WALLS][MAX_DLIST_TIME];
    int dispmode;
    int  sscal_;                  // selected scalar field
    int  reset;
    /////////////////
    // Color Editor
    /////////////////
    GLuint volren_graph[CAVE_MAX_WALLS];
    struct coloreditordata cedata;
    void check_grab();
    void update_held();
    void draw_coloreditor();


    ///////////////////
    // Volume Rendering
    ///////////////////
    float datasize[3];
    float data_origin[3];
    float dvertx[8][3];
    struct slicedata slicedata;

    void draw_volumedata();
    void setcolortable();

    void init_vertx();
    void make_slicedata();
    int  recalc_flag;
    void save_mindata(int, int);
  public:
    cave_volrend();
    int  get_recalc_flag(){return recalc_flag;};
    void initialize();
    void recalc(int n);
    void ani_recalc();
    void remake_voldata(int n);
    void turn_on() { this->on = 1; };
    void turn_off() { this->on = 0; };
    void plane_calc();

    int  sscal() {return this->sscal_;};
    void select_scal(int i) { this->sscal_ = i; };

    int  mode_out();
    void button3_has_pressed();
    void button2_has_released();
    void button_being_pressed();
    void draw();
    void clean();
    int	 is_on(){return on;};
    void make_mindata(int t);
    void load_mindata(int t);

    void set_coltable(int , unsigned char col[256][4]);
    void writedown_ic(char [], char[], int);
};


/**************************************************************
*** Change Scale
***************************************************************/

class scalefHandle {
  int       on;
  Snapstate *snapstate;    // Save states when middle button is pressed.
  float     vert_level_bar_height, vert_level_bar_rad_max;
  float     vert_level_bar_rad(float);
  void      left_arrow();
  void      all_arrows(); 
  void      make_vert_level_bar();
  void      make_level_rings();
  void      ring(float);
  float     work_vmax, work_vmin, work_dval, work_aver, work_vari;
  float     work_level;   // The arrows indicates this level while
                          // the handle is on. When wand's button 
                          // is released, this level is sent to isosurf.
  float     vert_drad_dlevel;  // bar radius change accord to the level.
  float     current_scale;

public:
  scalefHandle();
  void      turn(int n) { n==1 ? on = 1 : on = 0; };
  void      wand_motion();
  void      draw_vertical_bar(float, float, float, float, float);
  float     current_scale_is(){return current_scale;};
  void      set_current_scale(float cs){current_scale = cs;};
  void      draw();
  void      snap();
  void      button_has_pressed();
  void      button_has_released();  
  void      button_being_pressed();
  void      button3_has_pressed();
};

class  AnimControl{
  int  current_time;
  bool dlist_mode;
  bool anim_on;
  int  time_of_start;
  int  *dmade4anim;
  bool all_set;
  int  num_threads;
  int  TimeOfLoadedData;
  int  prev_frame;
  float prev_time;
  int  recover_flag;
  float intval;
  int   ani_recalc;
  bool dataload;

  //GUI
  Snapstate *snapstate;    // Save states when middle button is pressed.
  float     vert_level_bar_height, vert_level_bar_rad_max;

  float     work_vmax, work_vmin, work_aver, work_dval;
  float     work_level;   // The arrows indicates this level while
                          // the handle is on. When wand's button 
                          // is released, this level is sent to isosurf.
  float     vert_drad_dlevel;  // bar radius change accord to the level.

  void make_vert_level_bar ();
  void left_arrow ();
  void all_arrows ();
  void wand_motion ();
  void draw_vertical_bar (float, float, float, float, float);
  float vert_level_bar_rad (float);
public:
  AnimControl(char *);
  int  ctime(){return current_time;};
  bool is_dlistmode(){return dlist_mode;};
  bool is_animon(){return anim_on;}
  int  get_numthreads(){return num_threads;};
  int  get_tloaddata(){return TimeOfLoadedData;}
  void start_anim(bool);
  void prepare_make_dlists();
  void forward_time();
  void make_dlists_flagon();
  void make_dlists4anim();
  void stop();
  void OneStepForward();
  void OneStepBackward();
  void recover1();
  void recover2();  
  int  is_completed(int i){return dmade4anim[i];}
  void writedown_ic(char [], int);
  void set_ctime4init(int n){ current_time = n;}
  int get_recover_flag(){return recover_flag;}
  ///GUI
  void button_has_pressed ();
  void button_has_released ();
  void button_being_pressed ();
  void draw();
};

class Recalc{
  int recalc_flag[CAVE_MAX_WALLS][6];
  // 0:iso, 1:ortho_i, 2:ortho_j, 3:ortho_k, 4:volumerendering, 5:tracer
  // menu.cpp, wand_button.cpp, anicont.cpp
public:
  Recalc();
  void recalc_volrend();
  void recalc_iso();
  void recalc_ortho(int);
  void recalc_tracer();
  
  void iso_flag_on();
  void vol_flag_on(int);
  void ortho_flag_on(int);
  void tracer_flag_on();

  void recalc_all();
};

class Info_display {
  Snapstate *snapstate;      // Save states when left button is pressed.
  char jikoku[100];
  int on;

  public:
  Info_display();
  void draw();
  void update();
  void turn(){on = 1-on;}
  void turn_on (){on = 1;};
  int is_on() {return on;}
};

///////////////////////////
// inline functions
//////////////////////////////////////////////////

inline float inl_d2r(float deg) { return(deg/180.0*PI); }
inline float inl_r2d(float rad) { return(rad/PI*180.0); }
inline int TOGGLE(int x) {return(x==0 ? 1 : 0);}      // 0 -> 1, 1 -> 0    

#include "functions.h"

